@@ -19,7 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from datetime import date, datetime, time, timedelta
+import datetime as dt
+import logging
 import signal
 import sys
 
@@ -35,6 +36,7 @@ from .base import Pane, Window
 from .editor import EventEditor, ExportDialog
 from .calendarwidget import CalendarWidget
 
+logger = logging.getLogger('khal')
 
 #  Overview of how this all meant to fit together:
 #
@@ -128,11 +130,11 @@ class DateHeader(SelectableText):
 
         weekday = day.strftime('%A')
         daystr = day.strftime(dtformat)
-        if day == date.today():
+        if day == dt.date.today():
             return 'Today ({}, {})'.format(weekday, daystr)
-        elif day == date.today() + timedelta(days=1):
+        elif day == dt.date.today() + dt.timedelta(days=1):
             return 'Tomorrow ({}, {})'.format(weekday, daystr)
-        elif day == date.today() - timedelta(days=1):
+        elif day == dt.date.today() - dt.timedelta(days=1):
             return 'Yesterday ({}, {})'.format(weekday, daystr)
 
         approx_delta = utils.relative_timedelta_str(day)
@@ -164,7 +166,7 @@ class U_Event(urwid.Text):
         :type event: khal.event.Event
         """
         if relative:
-            if isinstance(this_date, datetime) or not isinstance(this_date, date):
+            if isinstance(this_date, dt.datetime) or not isinstance(this_date, dt.date):
                 raise ValueError('`this_date` is of type `{}`, sould be '
                                  '`datetime.date`'.format(type(this_date)))
         self.event = event
@@ -277,9 +279,9 @@ class DListBox(EventListBox):
 
     def render(self, size, focus=False):
         if self._init:
-            while 'bottom' in self.ends_visible(size):
-                self.body._autoextend()
             self._init = False
+        while not isinstance(self.body, StaticDayWalker) and 'bottom' in self.ends_visible(size):
+            self.body._autoextend()
         return super().render(size, focus)
 
     def clean(self):
@@ -304,7 +306,7 @@ class DListBox(EventListBox):
             key = 'down'
 
         if key in self._conf['keybindings']['today']:
-            self.parent.calendar.base_widget.set_focus_date(date.today())
+            self.parent.calendar.base_widget.set_focus_date(dt.date.today())
 
         rval = super().keypress(size, key)
         self.clean()
@@ -396,8 +398,8 @@ class DayWalker(urwid.SimpleFocusListWalker):
         :type end: datetime.date
         :type bool: bool
         """
-        start = start.date() if isinstance(start, datetime) else start
-        end = end.date() if isinstance(end, datetime) else end
+        start = start.date() if isinstance(start, dt.datetime) else start
+        end = end.date() if isinstance(end, dt.datetime) else end
 
         if everything:
             start = self[0].date
@@ -417,8 +419,8 @@ class DayWalker(urwid.SimpleFocusListWalker):
         :type start: datetime.date
         :type end: datetime.date
         """
-        start = start.date() if isinstance(start, datetime) else start
-        end = end.date() if isinstance(end, datetime) else end
+        start = start.date() if isinstance(start, dt.datetime) else start
+        end = end.date() if isinstance(end, dt.datetime) else end
 
         if everything:
             start = self[0].date
@@ -430,7 +432,7 @@ class DayWalker(urwid.SimpleFocusListWalker):
         day = start
         while day <= end:
             self.update_events_ondate(day)
-            day += timedelta(days=1)
+            day += dt.timedelta(days=1)
 
     def update_date_line(self):
         for one in self:
@@ -446,7 +448,7 @@ class DayWalker(urwid.SimpleFocusListWalker):
         return super().set_focus(position)
 
     def _autoextend(self):
-        self._last_day += timedelta(days=1)
+        self._last_day += dt.timedelta(days=1)
         pile = self._get_events(self._last_day)
         self.append(pile)
 
@@ -456,7 +458,7 @@ class DayWalker(urwid.SimpleFocusListWalker):
         # render() method does not get called otherwise, and they would
         # be indicated as the currently selected date
         self[self.focus or 0].reset_style()
-        self._first_day -= timedelta(days=1)
+        self._first_day -= dt.timedelta(days=1)
         pile = self._get_events(self._first_day)
         self.insert(0, pile)
 
@@ -505,7 +507,7 @@ class StaticDayWalker(DayWalker):
         num_days = max(1, self._conf['default']['timedelta'].days)
 
         for delta in range(num_days):
-            pile = self._get_events(day + timedelta(days=delta))
+            pile = self._get_events(day + dt.timedelta(days=delta))
             if len(self) <= delta:
                 self.append(pile)
             else:
@@ -535,8 +537,8 @@ class StaticDayWalker(DayWalker):
         :type start: datetime.date
         :type end: datetime.date
         """
-        start = start.date() if isinstance(start, datetime) else start
-        end = end.date() if isinstance(end, datetime) else end
+        start = start.date() if isinstance(start, dt.datetime) else start
+        end = end.date() if isinstance(end, dt.datetime) else end
 
         update = everything
         for one in self:
@@ -567,15 +569,15 @@ class DateListBox(NListBox):
 
     def render(self, size, focus):
         if focus:
-            self.body[0].set_attr_map({None: 'date focused'})
+            self.body[0].set_attr_map({None: 'date header focused'})
         elif DateListBox.selected_date == self.date:
-            self.body[0].set_attr_map({None: 'date selected'})
+            self.body[0].set_attr_map({None: 'date header selected'})
         else:
             self.reset_style()
         return super().render(size, focus)
 
     def reset_style(self):
-        self.body[0].set_attr_map({None: 'date'})
+        self.body[0].set_attr_map({None: 'date header'})
 
     def set_selected_date(self, day):
         """Mark `day` as selected
@@ -694,11 +696,11 @@ class EventColumn(urwid.WidgetWrap):
                 ('alert', 'Calendar `{}` is read-only.'.format(event.calendar)))
             return
 
-        if isinstance(event.start_local, datetime):
+        if isinstance(event.start_local, dt.datetime):
             original_start = event.start_local.date()
         else:
             original_start = event.start_local
-        if isinstance(event.end_local, datetime):
+        if isinstance(event.end_local, dt.datetime):
             original_end = event.end_local.date()
         else:
             original_end = event.end_local
@@ -714,9 +716,9 @@ class EventColumn(urwid.WidgetWrap):
             """
             # TODO cleverer support for recurring events, where more than start and
             # end dates are affected (complicated)
-            if isinstance(new_start, datetime):
+            if isinstance(new_start, dt.datetime):
                 new_start = new_start.date()
-            if isinstance(new_end, datetime):
+            if isinstance(new_end, dt.datetime):
                 new_end = new_end.date()
             start = min(original_start, new_start)
             end = max(original_end, new_end)
@@ -759,7 +761,7 @@ class EventColumn(urwid.WidgetWrap):
             new_pane = urwid.Columns([
                 ('weight', 2, ContainerWidget(editor)),
                 ('weight', 1, ContainerWidget(self.dlistbox))
-            ], dividechars=0, focus_column=0)
+            ], dividechars=2, focus_column=0)
             new_pane.title = editor.title
 
             def teardown(data):
@@ -848,9 +850,9 @@ class EventColumn(urwid.WidgetWrap):
                 self.pane.collection.writable_names[0]
             self.edit(event, always_save=True)
         start_date, end_date = event.start_local, event.end_local
-        if isinstance(start_date, datetime):
+        if isinstance(start_date, dt.datetime):
             start_date = start_date.date()
-        if isinstance(end_date, datetime):
+        if isinstance(end_date, dt.datetime):
             end_date = end_date.date()
         self.pane.eventscolumn.base_widget.update(start_date, end_date, event.recurring)
         try:
@@ -870,16 +872,16 @@ class EventColumn(urwid.WidgetWrap):
             self.pane.window.alert(('alert', 'No writable calendar.'))
             return
         if end is None:
-            start = datetime.combine(date, time(datetime.now().hour))
-            end = start + timedelta(minutes=60)
+            start = dt.datetime.combine(date, dt.time(dt.datetime.now().hour))
+            end = start + dt.timedelta(minutes=60)
             event = utils.new_event(
-                dtstart=start, dtend=end, summary="new event",
+                dtstart=start, dtend=end, summary='',
                 timezone=self._conf['locale']['default_timezone'],
                 locale=self._conf['locale'],
             )
         else:
             event = utils.new_event(
-                dtstart=date, dtend=end + timedelta(days=1), summary="new event",
+                dtstart=date, dtend=end + dt.timedelta(days=1), summary='',
                 allday=True, locale=self._conf['locale'],
             )
         event = self.pane.collection.new_event(
@@ -1013,7 +1015,6 @@ class SearchDialog(urwid.WidgetWrap):
 
 
 class ClassicView(Pane):
-
     """default Pane for khal
 
     showing a CalendarWalker on the left and the eventList + eventviewer/editor
@@ -1034,8 +1035,8 @@ class ClassicView(Pane):
         else:
             Walker = StaticDayWalker
         daywalker = Walker(
-            date.today(), eventcolumn=self, conf=self._conf, delete_status=self.delete_status,
-            collection=self.collection,
+            dt.date.today(), eventcolumn=self, conf=self._conf,
+            delete_status=self.delete_status, collection=self.collection,
         )
         elistbox = DListBox(
             daywalker, parent=self, conf=self._conf,
@@ -1051,6 +1052,7 @@ class ClassicView(Pane):
             on_press={key: self.new_event for key in self._conf['keybindings']['new']},
             firstweekday=self._conf['locale']['firstweekday'],
             weeknumbers=self._conf['locale']['weeknumbers'],
+            monthdisplay=self._conf['view']['monthdisplay'],
             get_styles=collection.get_styles
         )
         if self._conf['view']['dynamic_days']:
@@ -1118,7 +1120,7 @@ class ClassicView(Pane):
     def _search(self, search_term):
         """search for events matching `search_term"""
         self.window.backtrack()
-        events = list(self.collection.search(search_term))
+        events = sorted(self.collection.search(search_term))
         event_list = []
         event_list.extend([
             urwid.AttrMap(
@@ -1151,7 +1153,7 @@ class ClassicView(Pane):
         rval = super(ClassicView, self).render(size, focus)
         if self.init:
             # starting with today's events
-            self.eventscolumn.current_date = date.today()
+            self.eventscolumn.current_date = dt.date.today()
             self.init = False
         return rval
 
@@ -1255,10 +1257,54 @@ def _add_calendar_colors(palette, collection):
 
 def start_pane(pane, callback, program_info='', quit_keys=['q']):
     """Open the user interface with the given initial pane."""
+
     frame = Window(
         footer=program_info + ' | {}: quit, ?: help'.format(quit_keys[0]),
         quit_keys=quit_keys,
     )
+
+    class LogPaneFormatter(logging.Formatter):
+        def get_prefix(self, level):
+            if level >= 50:
+                return 'CRITICAL'
+            if level >= 40:
+                return 'ERROR'
+            if level >= 30:
+                return 'WARNING'
+            if level >= 20:
+                return 'INFO'
+            else:
+                return 'DEBUG'
+
+        def format(self, record):
+            return (self.get_prefix(record.levelno) + ': ' + record.msg)
+
+    class HeaderFormatter(LogPaneFormatter):
+        def format(self, record):
+            return (
+                super().format(record)[:30] + '... '
+                '[Press `{}` to view log]'.format(pane._conf['keybindings']['log'][0])
+            )
+
+    class LogPaneHandler(logging.Handler):
+        def emit(self, record):
+            frame.log(self.format(record))
+
+    class LogHeaderHandler(logging.Handler):
+        def emit(self, record):
+            frame.alert(self.format(record))
+
+    if len(logger.handlers) > 0 and not isinstance(logger.handlers[-1], logging.FileHandler):
+        logger.handlers.pop()
+
+    pane_handler = LogPaneHandler()
+    pane_handler.setFormatter(LogPaneFormatter())
+    logger.addHandler(pane_handler)
+
+    header_handler = LogHeaderHandler()
+    header_handler.setFormatter(HeaderFormatter())
+    logger.addHandler(header_handler)
+
     frame.open(pane, callback)
     palette = _add_calendar_colors(
         getattr(colors, pane._conf['view']['theme']), pane.collection)
@@ -1270,10 +1316,10 @@ def start_pane(pane, callback, program_info='', quit_keys=['q']):
         # XXX TODO this currently assumes, today moves forward by exactly one
         # day, but it could either move forward more (suspend-to-disk/ram) or
         # even move backwards
-        today = date.today()
+        today = dt.date.today()
         if meta['last_today'] != today:
             meta['last_today'] = today
-            pane.calendar.original_widget.reset_styles_range(today - timedelta(days=1), today)
+            pane.calendar.original_widget.reset_styles_range(today - dt.timedelta(days=1), today)
             pane.eventscolumn.original_widget.update_date_line()
         loop.set_alarm_in(60, redraw_today, pane)
 
