@@ -25,13 +25,12 @@ if anything doesn't work as expected, please open an issue for khal
 """
 
 import calendar
+import datetime as dt
 from collections import defaultdict
-from datetime import date
-from locale import getlocale, setlocale, LC_ALL, LC_TIME
-
-from khal.utils import get_month_abbr_len
+from locale import LC_ALL, LC_TIME, getlocale, setlocale
 
 import urwid
+from khal.utils import get_month_abbr_len
 
 setlocale(LC_ALL, '')
 
@@ -43,7 +42,7 @@ def getweeknumber(day):
     :return: weeknumber
     :rtype: int
     """
-    return date.isocalendar(day)[1]
+    return dt.date.isocalendar(day)[1]
 
 
 class DatePart(urwid.Text):
@@ -236,9 +235,9 @@ class CListBox(urwid.ListBox):
         super(CListBox, self).__init__(walker)
 
     def render(self, size, focus=False):
+        while 'bottom' in self.ends_visible(size):
+            self.body._autoextend()
         if self._init:
-            while 'bottom' in self.ends_visible(size):
-                self.body._autoextend()
             self.set_focus_valign('middle')
             self._init = False
 
@@ -313,9 +312,9 @@ class CListBox(urwid.ListBox):
 
     def keypress(self, size, key):
         if key in self.keybindings['mark'] + ['esc'] and self._marked:
-                self._unmark_all()
-                self._marked = False
-                return
+            self._unmark_all()
+            self._marked = False
+            return
         if key in self.keybindings['mark']:
             self._marked = {'date': self.body.focus_date,
                             'pos': (self.focus_position, self.focus.focus_col)}
@@ -338,7 +337,7 @@ class CListBox(urwid.ListBox):
             # reset colors of currently focused Date widget
             self.focus.focus.set_styles(self.focus.get_styles(self.body.focus_date, False))
         if key in self.keybindings['today']:
-            self.set_focus_date(date.today())
+            self.set_focus_date(dt.date.today())
             self.set_focus_valign(('relative', 10))
 
         key = super(CListBox, self).keypress(size, key)
@@ -349,11 +348,13 @@ class CListBox(urwid.ListBox):
 
 class CalendarWalker(urwid.SimpleFocusListWalker):
     def __init__(self, on_date_change, on_press, keybindings, firstweekday=0,
-                 weeknumbers=False, get_styles=None, initial=None):
+                 weeknumbers=False, monthdisplay='firstday', get_styles=None,
+                 initial=None):
         if initial is None:
-            initial = date.today()
+            initial = dt.date.today()
         self.firstweekday = firstweekday
         self.weeknumbers = weeknumbers
+        self.monthdisplay = monthdisplay
         self.on_date_change = on_date_change
         self.on_press = on_press
         self.keybindings = keybindings
@@ -484,7 +485,10 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
                   if today is in this week
         :rtype: tuple(urwid.CColumns, bool)
         """
-        if 1 in [day.day for day in week]:
+        if self.monthdisplay == 'firstday' and 1 in [day.day for day in week]:
+            month_name = calendar.month_abbr[week[-1].month].ljust(4)
+            attr = 'monthname'
+        elif self.monthdisplay == 'firstfullweek' and week[0].day <= 7:
             month_name = calendar.month_abbr[week[-1].month].ljust(4)
             attr = 'monthname'
         elif self.weeknumbers == 'left':
@@ -512,8 +516,8 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
         return week
 
     def _construct_month(self,
-                         year=date.today().year,
-                         month=date.today().month,
+                         year=dt.date.today().year,
+                         month=dt.date.today().month,
                          clean_first_row=False,
                          clean_last_row=False):
         """construct one month of DateCColumns
@@ -554,7 +558,7 @@ class CalendarWalker(urwid.SimpleFocusListWalker):
 
 class CalendarWidget(urwid.WidgetWrap):
     def __init__(self, on_date_change, keybindings, on_press, firstweekday=0,
-                 weeknumbers=False, get_styles=None, initial=None):
+                 weeknumbers=False, monthdisplay='firstday', get_styles=None, initial=None):
         """
         :param on_date_change: a function that is called every time the selected
             date is changed with the newly selected date as a first (and only
@@ -580,7 +584,7 @@ class CalendarWidget(urwid.WidgetWrap):
         :type on_press: dict
         """
         if initial is None:
-            self._initial = date.today()
+            self._initial = dt.date.today()
         else:
             self._initial = initial
 
@@ -605,12 +609,12 @@ class CalendarWidget(urwid.WidgetWrap):
 
         def _get_styles(date, focus):
             if focus:
-                if date == date.today():
+                if date == dt.date.today():
                     return 'today focus'
                 else:
                     return 'reveal focus'
             else:
-                if date == date.today():
+                if date == dt.date.today():
                     return 'today'
                 else:
                     return None
@@ -625,7 +629,8 @@ class CalendarWidget(urwid.WidgetWrap):
             [(2, urwid.AttrMap(urwid.Text(name), 'dayname')) for name in dnames],
             dividechars=1)
         self.walker = CalendarWalker(
-            on_date_change, on_press, default_keybindings, firstweekday, weeknumbers,
+            on_date_change, on_press, default_keybindings, firstweekday,
+            weeknumbers, monthdisplay,
             get_styles, initial=self._initial)
         self.box = CListBox(self.walker)
         frame = urwid.Frame(self.box, header=dnames)
@@ -633,7 +638,7 @@ class CalendarWidget(urwid.WidgetWrap):
         self.set_focus_date(self._initial)
 
     def focus_today(self):
-        self.set_focus_date(date.today())
+        self.set_focus_date(dt.date.today())
 
     def reset_styles_range(self, min_date, max_date):
         self.walker.reset_styles_range(min_date, max_date)
